/*
======================
    Define
======================
*/

#define MAX_LIGHTS 20
#define PI 3.14159265

/*
======================
    Vertex shader
======================
*/
varying vec3 f_normal;
varying vec3 v_normal;

#ifdef VERTEX
	attribute vec3 VertexNormal;
	attribute vec4 VertexWeight;
	attribute vec4 VertexBone; // NOTE: Actually used as int...
	
	uniform mat4 u_view, u_model, u_projection; // TODO: (ab)use love's built in matrices instead?
	uniform mat4 u_bone_matrices[200]; // this is why I want UBOs...

	uniform int	 u_skinning = 0;

	mat4 getDeformMatrix()
    {
		if (u_skinning != 0)
        {
			return u_bone_matrices[int(VertexBone.x)] * VertexWeight.x +
				   u_bone_matrices[int(VertexBone.y)] * VertexWeight.y +
				   u_bone_matrices[int(VertexBone.z)] * VertexWeight.z +
				   u_bone_matrices[int(VertexBone.w)] * VertexWeight.w;
		}
		return mat4(1.0);
	}

	vec4 position(mat4 mvp, vec4 v_position)
    {
        mat4 transform = u_model * getDeformMatrix();
		v_normal = VertexNormal;
		f_normal = (u_model * vec4(VertexNormal, 1.0)).xyz;
		return u_projection * u_view * transform * v_position;
	}
#endif

/*
======================
    Fragment shader
======================
*/
#ifdef PIXEL
	uniform vec3 ambient = vec3(0.0);
    
	vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords)
    {
		vec3 tex = texture2D(texture, texture_coords).rgb;
        
		return vec4(tex + ambient, 1.0);
	}
#endif